Lib.check = (function() {
    var obj = {};

    //PUBLIC FUNCTIONS.-------------------------------------
    //ALL public functions require arg check
    function exists(val) {
        return val !== undefined && val !== null;
    }
    obj.exists = exists;

    function isNumber(val) {
        if (!exists(val)) {
            return false;
        }
        return typeof val === "number" && !isNaN(val) && isFinite(val);
    }
    obj.isNumber = isNumber;

    function isString(val) {
        if (!exists(val)) {
            return false;
        }
        return typeof val === "string";
    }
    obj.isString = isString;

    function isFunction(val) {
        if (!exists(val)) {
            return false;
        }
        return typeof val === "function";
    }
    obj.isFunction = isFunction;

    function isObject(val) {
        if (!exists(val)) {
            return false;
        }
        return typeof val === "object";
    }
    obj.isObject = isObject;

    function isArray(val) {
        if (!exists(val) || !(val.constructor === Array)) {
            return false;
        } else {
            return true;
        }
    }
    obj.isArray = isArray;


    function isBoolean(val){
        if(!exists(val)){
            return false;
        }
        return typeof val === "boolean";
    }
    obj.isBoolean = isBoolean;


    //check if empty string. Passin other than string will return true (empty string);
    function isEmptyString(val) {
        if (!exists(val) || !isString(val)) {
            return true;
        }
        return val.length <= 0 || false;
    }
    obj.isEmptyString = isEmptyString;

    //check if objects are empty or undefined. Passing other than objects will return true (empty or undefined);
    function isEmptyOrUndefinedObj(val){

        if(!exists(val) || !isObject(val)){
            return true;
        }
        return Object.keys(val).length === 0 && val.constructor === Object;
    }
    obj.isEmptyOrUndefinedObj = isEmptyOrUndefinedObj;

    //Check if array is empty
    function isEmptyArray (val){
        if(!exists(val) || !isArray(val) || val.length <=0){
            return true;
        }
        return false;

    }
    obj.isEmptyArray = isEmptyArray;

    //do a boundary check and restrict val to min and max
    function boundary(min, max, val) {
        if (!totalArgCheck(arguments, 3, boundary)) {
            return 0;
        }

        if (val < min) {
            val = min;
        }
        if (val > max) {
            val = max;
        }
        return val;
    }

    obj.boundary = boundary;

    //if exists, if it's num returns va whithin boundaries. Otherwise default
    function totalNumCheck(min, max, defVal, val) {
        if (!theseArgsExist(min, max, defVal)) {
            p("They don't exist");
            return 0;
        }

        if (isNumber(val)) {

            return boundary(min, max, val);
        } else {
            return defVal;
        }
    }
    obj.totalNumCheck = totalNumCheck;



    //use it to see if number of min args are passed and that are all exist. Name of caller func is optional but recomended
    function totalArgCheck(arguments, numRequired, func, silent = false) {
        var funcName = func === undefined ? "unknown Function" : func.name;
        if (argLength(arguments) !== numRequired || !allArgsExist(arguments)) {

            if (!silent)
                Lib.glob.warn("arg length different than required or not all args are valid in " + funcName);

            return false;
        }
        return true;
    }
    obj.totalArgCheck = totalArgCheck;

    //
    function totalStringCheck(val, defVal) {
        // if (!totalArgCheck(arguments, 2,totalStringCheck,true)) {
        //     return "";
        // }

        if (isString(val) && !isEmptyString(val)) {
            return val;
        } else {
            return defVal;
        }

    }
    obj.totalStringCheck = totalStringCheck;

    //check for array existence, length and typeof
    function totalArrCheck(arr, reqType){
        if(isArray(arr) && isString(reqType)){
            if(arr.length === 0){
                Lib.glob.warn("Passed an empty array, returning false");
                return false;
            }

            for(var a =0; a< arr.length;a++){
                var val = arr[a];
                if(!(typeof val === reqType)){
                    return false;
                }
            }

        }
        else{
            Lib.glob.warn("INVALID ARGS - empty array or reqType is not string");
            return false;
        }

        return true;
    }

    obj.totalArrCheck =totalArrCheck;

    //variadic function
    function theseArgsExist() {

        if (argLength(arguments) === 0) {
            return false;
        }

        var argArray = Array.prototype.slice.call(arguments); //convert arguments obj to array

        for (arg in argArray) {
            if (!exists(argArray[arg])) {
                return false;
            }
        }
        return true;
    }

    obj.theseArgsExist = theseArgsExist;

    //property checker. Only checks for own props
    function objHasOwnProps(obj, propArray) {
        // if(!exists(obj) || !exists(propArray)){
        //     return false;
        // }
        if (!isObject(obj) || !isArray(propArray) || propArray.length === 0) {
            return false;
        }

        for (var a = 0; a < propArray.length; a++) {
            if (!isString(propArray[a])) {
                return false;
            }
            if (!obj.hasOwnProperty(propArray[a])) {
                return false;
            }
        }
        return true;
    }

    obj.objHasOwnProps = objHasOwnProps;


    //prop checker. Goes down the proto chain
    function objHasProps(obj,propArray){
        if (!isObject(obj) || !isArray(propArray) || propArray.length === 0) {
            return false;
        }

        for (var a = 0; a < propArray.length; a++) {

            if (!isString(propArray[a])) {
                return false;
            }
            if (!exists(obj[propArray[a]])) {
                return false;
            }
        }
        return true;

    }

    obj.objHasProps = objHasProps;


    // check if object has a method. Goes up the inheritance chain
    function objHasMethod(obj,methodName){
        if(!isObject(obj) || !isString(methodName || isEmptyString(methodName))){
            return false;
        }

        if(objHasProps(obj,[methodName]) && typeof obj[methodName] === "function"){
            return true;
        }

        return false;
    }

    obj.objHasMethod = objHasMethod;


    //check if array has value
    function arrHasVal(arr, val){
        if(!isArray(arr) || arr.length === 0 || !exists(val)){
            return false;
        }
        return arr.indexOf(val) > -1;
    }
    obj.arrHasVal = arrHasVal;

    //checks if a texture exists and is loaded in cache
    function isGameTexture(val){
        if(Lib.check.isEmptyString(val)){
            return false;
        }
        else{
            if(game.cache.checkImageKey(val)){
                return true;
            }
            return false;
        }
    }

    obj.isGameTexture = isGameTexture;
    //PRIVATE FUNCS---------------------------------------
    //########################################################
    function allArgsExist(arguments) {
        if (!exists(arguments)) {
            return false;
        }
        //convert arguments to an array
        var args = Array.prototype.slice.call(arguments);

        for (arg in args) {
            if (!exists(args[arg])) {
                return false;
            }
        }
        return true;
    }

    //-----------------
    function argLength(arguments) {
        if (!exists(arguments)) {
            return 0;
        }

        return arguments.length;
    }

    //#### END OF PRIVATE FUNCS ##############################################


    //--------------------
    return obj;
}());
